!! Purpose: Generating linked list from interactive input
!!          and
!!          printing it
!!
!! History: R. Barthel, Mar 2015, KIT, SCC

PROGRAM exercise2b_3
implicit none

!! Declaration of derived data type 
type node
  real                 :: val
  !! A pointer linking to a target must be of the same derived type
  !! therefore this intrinsic pointer is of type node
  type (node), pointer :: ptr_to_next
end type node
!!
!! Declare types that represent "positions" in linked list  
type (node), target  :: first_node
type (node), pointer :: curr_node, new_node
!!
!! Declare input variable, threshold
real                 :: r_input
real,parameter       :: threshold = 0.0

!! Point current position in list (val + ptr_to_next) on target "first
curr_node => first_node

!! Looping over interactive entries
gen_list: DO
   !!
   !! Reading in real 
   read *, r_input
   !! 
   !! Exit if input is below or equal threshold
   if (r_input .le. threshold) exit gen_list
   !!
   !! Allocate unnamed data object (val + ptr_to_next) 
   !! -> "new_node" is now pointing at it
   allocate(new_node)        !! intrinsic pointer and value at which "new_node" points are still undefined
   !!
   !! Write input to unnamed data object at which "new_node" points
   new_node%val = r_input
   !!
   !! Use undefined intrinsic pointer of data object at which 
   !! "curr_node" points and associated it to 
   !! unnamed data object at which currently "new_node" points
   curr_node%ptr_to_next => new_node
   !!
   !! Unalias curr_node pointing at old unnamed data object and set alias 
   !! to generated unnamed data object of this loop. 
   !! -> basically generating resetting "position of "curr_node" to latest 
   !!    data object
   curr_node => new_node
end do gen_list
!!
!! After terminating gen_list loop 
!! last intrinsic pointer (i.e. of curr_node) has to be disassociated
NULLIFY (curr_node%ptr_to_next)


!! Now for output we are setting our "position" to the beginning of the list
!! -> We require the link to first generated unnamed data object
!! -> This info is stored in the "zero-th" data object pointer
curr_node => first_node%ptr_to_next
!!
!! Looping by following the links 
printnode: do
  !! 
  !! Terminating loop when nullified pointer of last data object is found
  if (.not. ASSOCIATED(curr_node)) exit
  !!
  !! Printing value stored in "curr_node" pointing at currend unnamed data object
  write(*,*) curr_node%val
  !!
  !! Set link of "curr_node" to next unnamed data object at which the intrinsic pointer
  !! of the current data object points
  curr_node => curr_node%ptr_to_next
end do printnode


End Program exercise2b_3
